package at.ac.tuwien.dbai.ges.instances;

import java.util.*;

/**
 * Provides various ways to generate random numbers and the possibility to choose a seed.
 *
 * @author Lucas Kletzander
 */
public class RandomProvider {

    private static Random random;

    public static void initialize() {
        random = new Random();
    }

    public static void initialize(int seed) {
        random = new Random(seed);
    }

    public static Random get() {
        return random;
    }

    public static int getInBounds(int lowerInclusive, int upperInclusive) {
        return get().nextInt(upperInclusive - lowerInclusive + 1) + lowerInclusive;
    }

    public static int getInBounds(int lowerInclusive, int upperInclusive, int step) {
        return (get().nextInt(upperInclusive - lowerInclusive + step) + lowerInclusive) / step * step;
    }

    public static double getDoubleInBounds(double lower, double upper) {
        return get().nextDouble() * (upper - lower) + lower;
    }

    @SuppressWarnings("unchecked")
    public static <T> T getFromCollection(Collection<T> collection) {
        return (T) collection.toArray()[get().nextInt(collection.size())];
    }

    public static <T> T getFromCollection(Collection<T> collection, T def) {
        return collection.size() == 0 ? def : getFromCollection(collection);
    }

    public static double[] initializeRandomArray(int length, double max) {

        double[] arr = new double[length];
        Arrays.fill(arr, -1);

        int index = get().nextInt(length);
        arr[index] = 1;
        arr[getRandomExcept(length, index)] = max;

        for (int i = 0; i < length; i++) {

            if (arr[i] == -1) {
                arr[i] = get().nextDouble() * (max - 1) + 1;
            }

        }

        return transformRandomArray(arr);

    }

    private static int getRandomExcept(int bound, int except) {

        if (bound == 1) {
            return 0;
        }

        int rand = except;
        while (rand == except) {
            rand = get().nextInt(bound);
        }

        return rand;

    }

    public static double[] transformRandomArray(double[] arr) {

        double current = 0;
        double[] res = Arrays.stream(arr).map(i -> i / Arrays.stream(arr).sum()).toArray();

        for (int i = 0; i < res.length; i++) {
            current += res[i];
            res[i] = current;
        }

        // ensure nothing beyond last element
        res[res.length - 1] = 1.1;

        return res;

    }

    public static int getElementFromRandomArray(double[] arr) {

        int result = Arrays.binarySearch(arr, get().nextDouble());
        return result < 0 ? -result - 1 : result;

    }

}
