package at.ac.tuwien.dbai.ges.instances;

import at.ac.tuwien.dbai.ges.instances.breaks.BreakConfigurationType;
import at.ac.tuwien.dbai.ges.instances.breaks.BreakTimeFractionConstraintType;
import at.ac.tuwien.dbai.ges.instances.employee.WorkloadRestrictionType;
import at.ac.tuwien.dbai.ges.instances.restriction.RestrictionType;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftInstanceConstraintType;
import at.ac.tuwien.dbai.ges.instances.tasks.TaskGenerationType;

import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Properties;

/**
 * Provides access to all parameters set in the config file.
 *
 * @author Lucas Kletzander
 */
public class ScheduleConfig {

    private int days = 7;
    private int history = 0;
    private int timeSlotLength = 15;
    private int shiftSlotLength = 60;

    private int shiftTypes = 3;
    private int shiftAvg = 3;
    private double shiftVariation = 2;
    private double shiftSequenceRegularity = .9;
    private double dayDifference = 0;

    private int minShiftStart = 3;
    private int maxShiftStart = 22;
    private int minShiftInstances = 1;
    private int maxShiftInstances = 3;
    private int shiftInstanceDifference = 3;
    private ShiftInstanceConstraintType shiftInstanceConstraintType = ShiftInstanceConstraintType.MIN_MAX;
    private int minShiftLength = 8;
    private int maxShiftLength = 10;

    private double shiftDefinitionTightness = 1;
    private double weekendFactor = .5;

    private int contracts = 1;
    private double contractVariation = 1;
    private int[] contractTimes = {2400};
    private int[] minOn = {3};
    private int[] maxOn = {6};
    private int[] minOff = {1};
    private int[] maxOff = {3};

    private RestrictionType timeRestriction = RestrictionType.LOCAL;
    private double timeRestrictionFactor = .9;
    private double workloadTightness = 1;
    private WorkloadRestrictionType workloadRestrictionType = WorkloadRestrictionType.MAX;

    private RestrictionType weekTimeRestriction = RestrictionType.NONE;
    private double weekTimeRestrictionFactor = .5;

    private RestrictionType patternRestriction = RestrictionType.LOCAL;
    private int patternRestrictionCount = 2;

    private int minShiftPreferences = 1;
    private int maxShiftPreferences = 5;
    private int minShiftPreferenceWeight = 1;
    private int maxShiftPreferenceWeight = 5;

    private int breakTypes = 3;
    private double lunchBreakFactor = .2;
    private int maxBreakLength = 4;
    private int minBreakDistance = 3;
    private int maxBreakDistance = 24;

    private BreakConfigurationType breakConfigurationType = BreakConfigurationType.PER_SHIFT_LENGTH;
    private BreakTimeFractionConstraintType breakTimeFractionConstraintType = BreakTimeFractionConstraintType.MAX;
    private double breakTimeFractionRestriction = .5;
    private double breakTimeFractionFocus = .5;

    private int skills = 5;
    private double skillMin = .3;
    private double skillMax = .8;
    private double skillDistribution = .5;

    private int taskTypes = 5;
    private TaskGenerationType taskGenerationType = TaskGenerationType.WITH_BREAKS_AND_LONG;
    private int minTaskLength = 1;
    private int maxTaskLength = 8;
    private double commonTaskFactor = .5;
    private int taskWindowRange = 4;
    private int maxReAcquaintance = 1;
    private int taskDistanceRange = 3;

    private DemandType demandType = DemandType.TASKS;
    private boolean demandUseMax = true;
    private int demandSkills = 0;
    private double demandSlack = 0;
    private int demandTimeBlock = -1;

    public int getDays() {
        return days;
    }

    public int getHistory() {
        return history;
    }

    public int getTimeSlotLength() {
        return timeSlotLength;
    }

    public int getShiftSlotLength() {
        return shiftSlotLength;
    }

    public int getShiftTypes() {
        return shiftTypes;
    }

    public int getShiftAvg() {
        return shiftAvg;
    }

    public double getShiftVariation() {
        return shiftVariation;
    }

    public double getShiftSequenceRegularity() {
        return shiftSequenceRegularity;
    }

    public double getDayDifference() {
        return dayDifference;
    }

    public int getMinShiftStart() {
        return minShiftStart;
    }

    public int getMaxShiftStart() {
        return maxShiftStart;
    }

    public int getMinShiftInstances() {
        return minShiftInstances;
    }

    public int getMaxShiftInstances() {
        return maxShiftInstances;
    }

    public int getShiftInstanceDifference() {
        return shiftInstanceDifference;
    }

    public ShiftInstanceConstraintType getShiftInstanceConstraintType() {
        return shiftInstanceConstraintType;
    }

    public int getMinShiftLength() {
        return minShiftLength;
    }

    public int getMaxShiftLength() {
        return maxShiftLength;
    }

    public double getShiftDefinitionTightness() {
        return shiftDefinitionTightness;
    }

    public double getWeekendFactor() {
        return weekendFactor;
    }

    public int getContracts() {
        return contracts;
    }

    public double getContractVariation() {
        return contractVariation;
    }

    public int[] getContractTimes() {
        return contractTimes;
    }

    public int[] getMinOn() {
        return minOn;
    }

    public int[] getMaxOn() {
        return maxOn;
    }

    public int[] getMinOff() {
        return minOff;
    }

    public int[] getMaxOff() {
        return maxOff;
    }

    public RestrictionType getTimeRestriction() {
        return timeRestriction;
    }

    public double getTimeRestrictionFactor() {
        return timeRestrictionFactor;
    }

    public double getWorkloadTightness() {
        return workloadTightness;
    }

    public WorkloadRestrictionType getWorkloadRestrictionType() {
        return workloadRestrictionType;
    }

    public RestrictionType getWeekTimeRestriction() {
        return weekTimeRestriction;
    }

    public double getWeekTimeRestrictionFactor() {
        return weekTimeRestrictionFactor;
    }

    public RestrictionType getPatternRestriction() {
        return patternRestriction;
    }

    public int getPatternRestrictionCount() {
        return patternRestrictionCount;
    }

    public int getMinShiftPreferences() {
        return minShiftPreferences;
    }

    public int getMaxShiftPreferences() {
        return maxShiftPreferences;
    }

    public int getMinShiftPreferenceWeight() {
        return minShiftPreferenceWeight;
    }

    public int getMaxShiftPreferenceWeight() {
        return maxShiftPreferenceWeight;
    }

    public int getBreakTypes() {
        return breakTypes;
    }

    public double getLunchBreakFactor() {
        return lunchBreakFactor;
    }

    public int getMaxBreakLength() {
        return maxBreakLength;
    }

    public int getMinBreakDistance() {
        return minBreakDistance;
    }

    public int getMaxBreakDistance() {
        return maxBreakDistance;
    }

    public BreakConfigurationType getBreakConfigurationType() {
        return breakConfigurationType;
    }

    public BreakTimeFractionConstraintType getBreakTimeFractionConstraintType() {
        return breakTimeFractionConstraintType;
    }

    public double getBreakTimeFractionRestriction() {
        return breakTimeFractionRestriction;
    }

    public double getBreakTimeFractionFocus() {
        return breakTimeFractionFocus;
    }

    public int getSkills() {
        return skills;
    }

    public double getSkillMin() {
        return skillMin;
    }

    public double getSkillMax() {
        return skillMax;
    }

    public double getSkillDistribution() {
        return skillDistribution;
    }

    public int getTaskTypes() {
        return taskTypes;
    }

    public TaskGenerationType getTaskGenerationType() {
        return taskGenerationType;
    }

    public int getMinTaskLength() {
        return minTaskLength;
    }

    public int getMaxTaskLength() {
        return maxTaskLength;
    }

    public double getCommonTaskFactor() {
        return commonTaskFactor;
    }

    public int getTaskWindowRange() {
        return taskWindowRange;
    }

    public int getMaxReAcquaintance() {
        return maxReAcquaintance;
    }

    public int getTaskDistanceRange() {
        return taskDistanceRange;
    }

    public DemandType getDemandType() {
        return demandType;
    }

    public boolean isDemandUseMax() {
        return demandUseMax;
    }

    public int getDemandSkills() {
        return demandSkills;
    }

    public double getDemandSlack() {
        return demandSlack;
    }

    public int getDemandTimeBlock() {
        return demandTimeBlock;
    }

    @SuppressWarnings("unchecked")
    public static ScheduleConfig load(String propertyPath) {

        ScheduleConfig config = new ScheduleConfig();

        try {

            Properties properties = new Properties();
            properties.load(new FileInputStream(propertyPath));

            System.out.println("Loading configuration...");

            for (Field field : ScheduleConfig.class.getDeclaredFields()) {

                if (properties.containsKey(field.getName())) {

                    String property = properties.getProperty(field.getName());

                    if (field.getType().equals(boolean.class)) {
                        field.setBoolean(config, Boolean.parseBoolean(property));
                    } else if (field.getType().equals(double.class)) {
                        field.setDouble(config, Double.parseDouble(property));
                    } else if (field.getType().equals(int.class)) {
                        field.setInt(config, Integer.parseInt(property));
                    } else if (field.getType().equals(int[].class)) {
                        field.set(config, Arrays.stream(property.split(";")).mapToInt(Integer::parseInt).toArray());
                    } else if (field.getType().isEnum()) {
                        field.set(config, Enum.valueOf((Class<? extends Enum>) field.getType(), property.toUpperCase()));
                    }

                }

            }

        } catch (IOException | IllegalAccessException e) {
            System.out.println("Error processing config file");
            e.printStackTrace();
        }

        return config;

    }

}
