package at.ac.tuwien.dbai.ges.instances.employee;

import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftDetails;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftGenerator;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftTypeDefinition;
import at.ac.tuwien.dbai.ges.instances.transformer.Transformer;
import at.ac.tuwien.dbai.ges.schema.EmployeeList;
import at.ac.tuwien.dbai.ges.schema.PreAssignments;

/**
 * Transforms the history for a specific employee.
 *
 * @author Lucas Kletzander
 */
public class HistoryConfiguration extends EmployeeConfiguration {

    private final ScheduleConfig config;
    private final ShiftGenerator shiftGenerator;

    public HistoryConfiguration(ScheduleConfig config, ShiftGenerator shiftGenerator) {
        this.config = config;
        this.shiftGenerator = shiftGenerator;
    }

    @Override
    public void transformForRow(EmployeeList.Employee employee, EmployeeSchedule schedule) {

        PreAssignments preAssignments = employee.getPreAssignments();
        if (preAssignments == null) {
            preAssignments = new PreAssignments();
            employee.setPreAssignments(preAssignments);
        }

        for (int i = 0; i < config.getHistory(); i++) {

            ShiftDetails details = schedule.getDetails().get(i);
            if (details == null) {
                continue;
            }

            PreAssignments.Shift shift = new PreAssignments.Shift();
            shift.setID("S" + shiftGenerator.getShiftType(details.getShiftInstance()));
            shift.setDay(String.valueOf(i - config.getHistory()));

            ShiftTypeDefinition definition = shiftGenerator.getDefinitions()[shiftGenerator.getShiftType(details.getShiftInstance())];
            shift.setStartTime(Transformer.transformTimePoint(definition.getMinStartTime()));
            shift.setEndTime(Transformer.transformTimePoint(definition.getMinEndTime()));

            preAssignments.getShiftOrNoShift().add(shift);

        }

    }

}
