package at.ac.tuwien.dbai.ges.instances.employee;

import at.ac.tuwien.dbai.ges.instances.RandomProvider;
import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftGenerator;
import at.ac.tuwien.dbai.ges.schema.AbstractEmployeeType;
import at.ac.tuwien.dbai.ges.schema.AnyShift;
import at.ac.tuwien.dbai.ges.schema.EmployeeList;

import java.util.ArrayList;
import java.util.List;

/**
 * Generates shift preferences for a specific employee.
 *
 * @author Lucas Kletzander
 */
public class ShiftPreference extends EmployeeConfiguration {

    private final ScheduleConfig config;
    private final ShiftGenerator shiftGenerator;

    public ShiftPreference(ScheduleConfig config, ShiftGenerator shiftGenerator) {
        this.config = config;
        this.shiftGenerator = shiftGenerator;
    }

    @Override
    public void transformForRow(EmployeeList.Employee employee, EmployeeSchedule schedule) {

        AbstractEmployeeType.Preferences preferences = employee.getPreferences();
        if (preferences == null) {
            preferences = new AbstractEmployeeType.Preferences();
            employee.setPreferences(preferences);
        }

        List<Object> list = preferences.getPenalizedTaskOrShiftOffRequestOrShiftOnRequest();

        int num = RandomProvider.getInBounds(config.getMinShiftPreferences(), config.getMaxShiftPreferences());
        List<Integer> days = new ArrayList<>();

        for (int i = 0; i < num; i++) {

            if (config.getDays() == days.size()) {
                break;
            }

            int cur;
            do {
                cur = RandomProvider.get().nextInt(config.getDays());
            } while (days.contains(cur));

            days.add(cur);
            int weight = RandomProvider.getInBounds(config.getMinShiftPreferenceWeight(), config.getMaxShiftPreferenceWeight());

            if (schedule.getDetails().get(cur + config.getHistory()) == null) {

                AbstractEmployeeType.Preferences.ShiftOffRequest shiftOffRequest = new AbstractEmployeeType.Preferences.ShiftOffRequest();
                shiftOffRequest.setWeight((double) weight);
                shiftOffRequest.setDay(String.valueOf(cur));
                shiftOffRequest.setAnyShift(new AnyShift());
                list.add(shiftOffRequest);

            } else {

                AbstractEmployeeType.Preferences.ShiftOnRequest shiftOnRequest = new AbstractEmployeeType.Preferences.ShiftOnRequest();
                shiftOnRequest.setWeight((double) weight);
                shiftOnRequest.setDay(String.valueOf(cur));
                shiftOnRequest.setShiftTypes("S" + shiftGenerator.getShiftType(schedule.getDetails().get(cur + config.getHistory()).getShiftInstance()));
                list.add(shiftOnRequest);

            }

        }

    }

}
