package at.ac.tuwien.dbai.ges.instances.employee;

import at.ac.tuwien.dbai.ges.instances.RandomProvider;
import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.IntStream;

/**
 * Generates skills for employees.
 *
 * @author Lucas Kletzander
 */
public class SkillGenerator {

    private final double[] probabilities;
    private final double factor;

    public SkillGenerator(ScheduleConfig config) {

        this.probabilities = new double[config.getSkills()];
        Arrays.setAll(probabilities, i -> RandomProvider.getDoubleInBounds(config.getSkillMin(), config.getSkillMax()));
        this.factor = config.getSkillDistribution();

    }

    public int getSkills() {
        return probabilities.length;
    }

    public List<Integer> generateEmployeeSkills() {

        List<Integer> skills = new ArrayList<>();
        boolean notSkilled = RandomProvider.get().nextBoolean();

        IntStream.range(0, probabilities.length).filter(i -> {
            double thresholdFactor = RandomProvider.getDoubleInBounds(factor, 1);
            return RandomProvider.get().nextDouble() < (notSkilled ? probabilities[i] * thresholdFactor : 1 - (1 - probabilities[i]) * thresholdFactor);
        }).forEach(skills::add);

        return getSkills() > 0 && skills.isEmpty() ? generateEmployeeSkills() : skills;

    }

}
