package at.ac.tuwien.dbai.ges.instances.restriction;

import at.ac.tuwien.dbai.ges.instances.shift.ShiftRow;
import at.ac.tuwien.dbai.ges.schema.Contracts;
import at.ac.tuwien.dbai.ges.schema.ObjectFactory;

import java.util.List;

/**
 * Models a restriction that can either be applied globally of for each contract.
 *
 * @author Lucas Kletzander
 */
public abstract class Restriction {

    private final boolean global;
    private List<List<ShiftRow>> rows;

    protected Restriction(boolean global) {
        this.global = global;
    }

    public void setRows(List<List<ShiftRow>> rows) {
        this.rows = rows;
    }

    public void applyRestriction() {

        if (global) {

            initResult(1);
            initEvaluationData();
            rows.forEach(i -> i.forEach(this::evaluateRow));
            performRestriction(0);
            rows.forEach(i -> i.removeIf(j -> checkRemove(j, 0)));

        } else {

            initResult(rows.size());

            for (int i = 0; i < rows.size(); i++) {

                initEvaluationData();
                rows.get(i).forEach(this::evaluateRow);
                performRestriction(i);
                int finalI = i;
                rows.get(i).removeIf(j -> checkRemove(j, finalI));

            }

        }

    }

    protected abstract void initResult(int size);

    protected abstract void initEvaluationData();

    protected abstract void evaluateRow(ShiftRow row);

    protected abstract void performRestriction(int contract);

    protected abstract boolean checkRemove(ShiftRow row, int contract);

    public boolean checkRemove(ShiftRow row) {
        return checkRemove(row, global ? 0 : row.getContract());
    }

    public abstract long getFactor();

    public void transform(ObjectFactory objectFactory, Contracts.Contract contract) {
        transform(objectFactory, contract, global ? 0 : Integer.parseInt(contract.getID().substring(1)));
    }

    protected abstract void transform(ObjectFactory objectFactory, Contracts.Contract c, int contract);

}
