package at.ac.tuwien.dbai.ges.instances.restriction;

import at.ac.tuwien.dbai.ges.instances.shift.ShiftRow;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

/**
 * Tracks and applies restrictions to shift rows.
 *
 * @author Lucas Kletzander
 */
public class RestrictionMonitor {

    private final List<List<ShiftRow>> rows;
    private final List<Restriction> restrictions;
    private int rowCount;
    private int curRestriction;

    public RestrictionMonitor(int contracts) {

        rows = new ArrayList<>();
        IntStream.range(0, contracts).forEach(i -> rows.add(new ArrayList<>()));
        restrictions = new ArrayList<>();
        rowCount = 0;
        curRestriction = 0;

    }

    public void addRestriction(Restriction restriction) {
        restrictions.add(restriction);
    }

    public void addRow(ShiftRow row) {

        for (int i = 0; i < curRestriction; i++) {
            if (restrictions.get(i).checkRemove(row)) {
                return;
            }
        }

        rows.get(row.getContract()).add(row);
        rowCount++;

        if (rowCount > 100000) {
            applyNextRestriction();
        }

    }

    public List<List<ShiftRow>> getRows() {
        return rows;
    }

    public void applyRestrictions() {

        while (applyNextRestriction());

    }

    private boolean applyNextRestriction() {

        if (curRestriction < restrictions.size()) {

            System.out.println("Starting restriction with count " + rowCount + "...");

            Restriction restriction = restrictions.get(curRestriction++);
            restriction.setRows(rows);
            restriction.applyRestriction();

            rowCount = rows.stream().mapToInt(List::size).sum();
            System.out.println("Restriction application reduced count to " + rowCount);

            return true;

        }

        return false;

    }

    public long getFactor() {
        return restrictions.stream().mapToLong(Restriction::getFactor).reduce(1, (j, k) -> j * k);
    }

    public List<Restriction> getRestrictions() {
        return restrictions;
    }

}
