package at.ac.tuwien.dbai.ges.instances.shift;

import at.ac.tuwien.dbai.ges.instances.breaks.Break;
import at.ac.tuwien.dbai.ges.instances.breaks.BreakConfiguration;
import at.ac.tuwien.dbai.ges.instances.tasks.Task;

import java.util.ArrayList;
import java.util.List;

/**
 * Models the detailed schedule for one shift.
 *
 * @author Lucas Kletzander
 */
public class ShiftDetails {

    private final int shiftInstance;
    private final int startTime;
    private final int duration;
    private final int breakConfig;
    private List<Break> breaks;
    private List<Integer> lunchBreaks;
    private double breakTimeFraction;
    private List<Task> tasks;

    public ShiftDetails(ShiftInstance instance, int index) {

        shiftInstance = index;
        startTime = instance.getStartTimeNative();
        duration = instance.getDuration();
        this.breakConfig = -1;
        breaks = new ArrayList<>();
        tasks = new ArrayList<>();

    }

    public ShiftDetails(ShiftInstance instance, int index, int breakConfig) {

        shiftInstance = index;
        startTime = instance.getStartTimeNative();
        duration = instance.getDuration();
        this.breakConfig = breakConfig;
        breaks = new ArrayList<>();
        tasks = new ArrayList<>();

    }

    public ShiftDetails(ShiftDetails details) {

        shiftInstance = details.shiftInstance;
        startTime = details.startTime;
        duration = details.duration;
        breakConfig = details.breakConfig;
        breaks = new ArrayList<>(details.breaks);
        lunchBreaks = new ArrayList<>(details.lunchBreaks);
        breakTimeFraction = details.breakTimeFraction;
        tasks = new ArrayList<>(details.tasks);

    }

    public void setBreakResults(List<Integer> lunchBreaks, double breakTimeFraction) {

        this.lunchBreaks = lunchBreaks;
        this.breakTimeFraction = breakTimeFraction;

    }

    public void setUsed(BreakConfiguration config) {

        lunchBreaks.forEach(config::setTypeUnused);
        config.addWorkAfterLast(startTime + duration - breaks.get(breaks.size() - 1).getEnd());
        config.addBreakTimeFraction(breakTimeFraction);

    }

    public int getShiftInstance() {
        return shiftInstance;
    }

    public int getStartTime() {
        return startTime;
    }

    public int getDuration() {
        return duration;
    }

    public int getEndTime() {
        return startTime + duration;
    }

    public int getBreakConfig() {
        return breakConfig;
    }

    public List<Break> getBreaks() {
        return breaks;
    }

    public List<Task> getTasks() {
        return tasks;
    }

    public double getBreakTimeFraction() {
        return breakTimeFraction;
    }

    @Override
    public String toString() {
        return "ShiftDetails{" +
                "shiftInstance=" + shiftInstance +
                ", startTime=" + startTime +
                ", duration=" + duration +
                ", breakConfig=" + breakConfig +
                ", breaks=" + breaks +
                ", breakTimeFraction=" + breakTimeFraction +
                ", tasks=" + tasks +
                '}';
    }

}
