package at.ac.tuwien.dbai.ges.instances.shift;

import at.ac.tuwien.dbai.ges.instances.RandomProvider;

/**
 * Specifies a shift type.
 *
 * @author Lucas Kletzander
 */
public class ShiftTypeDefinition {

    private final int intervalLength;
    private final double tightness;

    private int minStartTime = Integer.MAX_VALUE;
    private int maxStartTime = 0;
    private int minEndTime = Integer.MAX_VALUE;
    private int maxEndTime = 0;
    private int instances = 0;

    public ShiftTypeDefinition(int intervalLength, double tightness) {

        this.intervalLength = intervalLength;
        this.tightness = tightness;

    }

    public void addShift(ShiftInstance shiftInstance) {

        minStartTime = Math.min(minStartTime, shiftInstance.getStartTimeNative());
        maxStartTime = Math.max(maxStartTime, shiftInstance.getStartTimeNative());
        minEndTime = Math.min(minEndTime, shiftInstance.getEndTimeNative());
        maxEndTime = Math.max(maxEndTime, shiftInstance.getEndTimeNative());
        instances++;

    }

    public void applyTightness() {

        double startDifference = 2 * (maxStartTime - minStartTime) * (1 - tightness);
        if (startDifference == 0) {
            startDifference = 5 * intervalLength * (1 - tightness);
        }
        double focus = RandomProvider.get().nextDouble();

        minStartTime = (int) (Math.max(0, minStartTime - startDifference * focus) / intervalLength * intervalLength);
        maxStartTime = (int) ((maxStartTime + startDifference * (1 - focus)) / intervalLength * intervalLength);

        double endDifference = 2 * (maxEndTime - minEndTime) * (1 - tightness);
        if (endDifference == 0) {
            endDifference = 5 * intervalLength * (1 - tightness);
        }
        focus = RandomProvider.get().nextDouble();

        minEndTime = (int) (Math.max(0, minEndTime - endDifference * focus) / intervalLength * intervalLength);
        maxEndTime = (int) ((maxEndTime + endDifference * (1 - focus)) / intervalLength * intervalLength);

    }

    public int getMinStartTime() {
        return minStartTime;
    }

    public int getMaxStartTime() {
        return maxStartTime;
    }

    public int getMinEndTime() {
        return minEndTime;
    }

    public int getMaxEndTime() {
        return maxEndTime;
    }

    public int getInstances() {
        return instances;
    }

    @Override
    public String toString() {
        return "ShiftTypeDefinition{" +
                "minStartTime=" + (minStartTime / 60) + ":" + (minStartTime % 60) +
                ", maxStartTime=" + (maxStartTime / 60) + ":" + (maxStartTime % 60) +
                ", minEndTime=" + (minEndTime / 60) + ":" + (minEndTime % 60) +
                ", maxEndTime=" + (maxEndTime / 60) + ":" + (maxEndTime % 60) +
                '}';
    }

}
