package at.ac.tuwien.dbai.ges.instances.tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

/**
 * Defines a task possibly split into several blocks of work.
 *
 * @author Lucas Kletzander
 */
public class Task {

    private final int type;
    private final int skill;
    private final List<Integer> starts;
    private final List<Integer> ends;
    private int day;
    private int usage;
    private int id;
    private int windowBefore;
    private int windowAfter;

    public Task(int type, int skill) {

        this.type = type;
        this.skill = skill;
        starts = new ArrayList<>();
        ends = new ArrayList<>();
        usage = 0;

    }

    public int getType() {
        return type;
    }

    public int getSkill() {
        return skill;
    }

    public void add(int day, int start, int end) {

        if (starts.isEmpty()) {

            while (start >= 24 * 60) {
                day++;
                start -= 24 * 60;
                end -= 24 * 60;
            }

            this.day = day;

        }

        if (!ends.isEmpty() && ends.get(ends.size() - 1) == start - (this.day - day) * 24 * 60) {

            ends.set(ends.size() - 1, end - (this.day - day) * 24 * 60);

        } else {

            starts.add(start - (this.day - day) * 24 * 60);
            ends.add(end - (this.day - day) * 24 * 60);

        }

    }

    public int getStartGlobal() {
        return day * 24 * 60 + getStart(day);
    }

    public int getDay() {
        return day;
    }

    public int getStart(int day) {
        return (this.day - day) * 24 * 60 + starts.get(0);
    }

    public int getEnd(int day) {
        return (this.day - day) * 24 * 60 + ends.get(ends.size() - 1);
    }

    public List<Integer> getStartList() {
        return starts;
    }

    public List<Integer> getEndList() {
        return ends;
    }

    public int getLength() {
        return IntStream.range(0, starts.size()).map(i -> ends.get(i) - starts.get(i)).sum();
    }

    public boolean addUsage() {

        usage++;
        return usage == 1;

    }

    public int getUsage() {
        return usage;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public int getWindowBefore() {
        return windowBefore;
    }

    public int getWindowAfter() {
        return windowAfter;
    }

    public int getStartGlobalWithWindow() {
        return day * 24 * 60 + getStart(day) - windowBefore;
    }

    public void setWindow(int windowBefore, int windowAfter) {
        this.windowBefore = windowBefore;
        this.windowAfter = windowAfter;
    }

    @Override
    public String toString() {
        return "Task{" +
                "type=" + type +
                ", skill=" + skill +
                ", starts=" + starts +
                ", ends=" + ends +
                ", day=" + day +
                ", usage=" + usage +
                '}';
    }

}
