package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.instances.DemandType;
import at.ac.tuwien.dbai.ges.instances.ScheduleGenerator;
import at.ac.tuwien.dbai.ges.schema.General;
import at.ac.tuwien.dbai.ges.schema.ObjectFactory;
import at.ac.tuwien.dbai.ges.schema.SchedulingHorizon;

import java.math.BigInteger;

/**
 * Transforms the problem description to the XML format.
 *
 * @author Lucas Kletzander
 */
public class HorizonTransformer extends Transformer<SchedulingHorizon> {

    private final ScheduleGenerator generator;
    private final SchedulingHorizon horizon;
    private final ObjectFactory objectFactory;

    public HorizonTransformer(ScheduleGenerator generator) {

        this.generator = generator;
        this.horizon = new SchedulingHorizon();
        objectFactory = new ObjectFactory();

        General general = new General();
        general.setPeriodLength(BigInteger.valueOf(generator.getConfig().getDays()));
        general.setTimeSlotLength(String.valueOf(generator.getConfig().getTimeSlotLength()));
        general.setShiftDesign(generator.getConfig().getMaxShiftInstances() > 1);
        General.AllowSequenceCutoff cutoff = new General.AllowSequenceCutoff();
        cutoff.setHorizonStart(true);
        cutoff.setHorizonEnd(true);
        general.setAllowSequenceCutoff(cutoff);
        horizon.setGeneral(general);

    }

    @Override
    public void transform() {

        ShiftTransformer shiftTransformer = new ShiftTransformer(generator.getConfig(), objectFactory);
        horizon.setShifts(shiftTransformer.transform(generator.getShiftGenerator().getDefinitions()));

        EmployeeTransformer employeeTransformer = new EmployeeTransformer(generator.getConfig(), objectFactory);
        horizon.setEmployees(employeeTransformer.transform(generator.getData(), generator.getRestrictionMonitor().getRestrictions(), generator.getEmployeeMonitor(), generator.getSkillGenerator() == null ? 0 : generator.getSkillGenerator().getSkills()));

        if (generator.getConfig().getDemandType() == DemandType.SHIFTS) {

            ShiftDemandTransformer shiftDemandTransformer = new ShiftDemandTransformer(generator.getConfig(), objectFactory);
            horizon.setDemands(shiftDemandTransformer.transformDemands(generator.getShiftGenerator(), generator.getData()));

        } else {

            BreakTransformer breakTransformer = new BreakTransformer(generator.getConfig(), objectFactory);
            horizon.setBreaks(breakTransformer.transform(generator.getBreakGenerator()));

            if (generator.getConfig().getDemandType() == DemandType.TIME) {

                TimeTransformer timeTransformer = new TimeTransformer(generator.getConfig(), objectFactory);
                horizon.setDemands(timeTransformer.transformDemands(generator.getData()));

            } else {

                TaskTransformer taskTransformer = new TaskTransformer(generator.getConfig(), objectFactory);
                horizon.setTasks(taskTransformer.transformTypes(generator.getTaskGenerator().getReAc()));
                horizon.setDemands(taskTransformer.transformDemands(generator.getTaskGenerator().getTasks()));

            }

        }

    }

    @Override
    protected SchedulingHorizon getRoot() {
        return horizon;
    }

}
