package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.instances.RandomProvider;
import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;
import at.ac.tuwien.dbai.ges.instances.employee.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftDetails;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftGenerator;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftRow;
import at.ac.tuwien.dbai.ges.schema.*;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Transforms the problem to the shift demand specification.
 *
 * @author Lucas Kletzander
 */
public class ShiftDemandTransformer {

    private final ScheduleConfig config;
    private final ObjectFactory objectFactory;

    public ShiftDemandTransformer(ScheduleConfig config, ObjectFactory objectFactory) {
        this.config = config;
        this.objectFactory = objectFactory;
    }

    public Demands transformDemands(ShiftGenerator shiftGenerator, List<EmployeeSchedule> schedules) {

        int[][] count = new int[config.getDays()][config.getShiftTypes()];

        for (EmployeeSchedule schedule : schedules) {

            for (int i = 0; i < schedule.getDetails().size() - config.getHistory(); i++) {

                ShiftDetails details = schedule.getDetails().get(i + config.getHistory());

                if (details != null) {
                    count[i][shiftGenerator.getShiftType(details.getShiftInstance())]++;
                }

            }

        }

        Demands demands = new Demands();

        for (int i = 0; i < config.getDays(); i++) {

            for (int j = 0; j < config.getShiftTypes(); j++) {

                ShiftCover cover = new ShiftCover();
                cover.setID("SC" + i);
                cover.setShift("S" + j);
                cover.setDay(String.valueOf(i));

                CoverRequirements requirements = new CoverRequirements();

                int minSlack = (int) (RandomProvider.get().nextDouble() * config.getDemandSlack() * count[i][j]);
                requirements.getMinOrMax().add(objectFactory.createCoverRequirementsMin(Transformer.transformIntSkill(Math.max(count[i][j] - minSlack, 0))));

                if (config.isDemandUseMax()) {
                    requirements.getMinOrMax().add(objectFactory.createCoverRequirementsMax(Transformer.transformIntSkill(count[i][j] + (int) (RandomProvider.get().nextDouble() * config.getDemandSlack() * (count[i][j] + 1)))));
                }

                cover.setRequirements(requirements);
                demands.getShiftCover().add(cover);

            }

        }

        return demands;

    }

}
