package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftInstanceConstraintType;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftTypeDefinition;
import at.ac.tuwien.dbai.ges.schema.*;

import javax.xml.bind.JAXBElement;
import java.util.List;

/**
 * Transforms the shift section of the problem to XML.
 *
 * @author Lucas Kletzander
 */
public class ShiftTransformer {

    private final ScheduleConfig config;
    private final ObjectFactory objectFactory;

    public ShiftTransformer(ScheduleConfig config, ObjectFactory objectFactory) {

        this.config = config;
        this.objectFactory = objectFactory;

    }

    public Shifts transform(ShiftTypeDefinition definitions[]) {

        Shifts shifts = new Shifts();
        ShiftTypes shiftTypes = new ShiftTypes();

        for (int i = 0; i < definitions.length; i++) {

            ShiftTypes.ShiftType shiftType = new ShiftTypes.ShiftType();
            shiftType.setID("S" + i);
            shiftType.setName("ShiftType " + i);
            shiftType.setLabel("S" + i);
            List<JAXBElement<?>> times = shiftType.getMinStartTimeOrMaxStartTimeOrMinEndTime();
            times.add(objectFactory.createShiftTypesShiftTypeMinStartTime(Transformer.transformWeightedTimePoint(definitions[i].getMinStartTime())));
            times.add(objectFactory.createShiftTypesShiftTypeMaxStartTime(Transformer.transformWeightedTimePoint(definitions[i].getMaxStartTime())));
            times.add(objectFactory.createShiftTypesShiftTypeMinEndTime(Transformer.transformWeightedTimePoint(definitions[i].getMinEndTime())));
            times.add(objectFactory.createShiftTypesShiftTypeMaxEndTime(Transformer.transformWeightedTimePoint(definitions[i].getMaxEndTime())));
            shiftTypes.getShiftType().add(shiftType);

        }

        shifts.setShiftTypes(shiftTypes);
        addShiftInstanceConstraint(shifts, definitions);
        shifts.setAllowBorderShifts(true);
        shifts.setTimeSlotLength(String.valueOf(config.getShiftSlotLength()));

        return shifts;

    }

    private void addShiftInstanceConstraint(Shifts shifts, ShiftTypeDefinition[] definitions) {

        if (config.getShiftInstanceConstraintType() == ShiftInstanceConstraintType.NONE) {
            return;
        }

        GlobalShiftConstraints constraints = new GlobalShiftConstraints();

        int minInstances = Integer.MAX_VALUE;
        int maxInstances = 0;
        int sumInstances = 0;

        for (int i = 0; i < definitions.length; i++) {

            if (config.getShiftInstanceConstraintType() == ShiftInstanceConstraintType.PER_TYPE_MAX) {

                GlobalShiftConstraints.ShiftInstances instances = new GlobalShiftConstraints.ShiftInstances();
                instances.setShiftTypes("S" + i);
                List<JAXBElement<WeightedInteger>> minMax = instances.getMinOrMax();
                minMax.add(objectFactory.createGlobalShiftConstraintsShiftInstancesMax(Transformer.transformInteger(definitions[i].getInstances())));
                constraints.getAverageShiftLengthOrShiftInstances().add(instances);

            }

            minInstances = Math.min(minInstances, definitions[i].getInstances());
            maxInstances = Math.max(maxInstances, definitions[i].getInstances());
            sumInstances += definitions[i].getInstances();

        }

        if (config.getShiftInstanceConstraintType() == ShiftInstanceConstraintType.MIN_MAX) {

            for (int i = 0; i < definitions.length; i++) {

                GlobalShiftConstraints.ShiftInstances instances = new GlobalShiftConstraints.ShiftInstances();
                instances.setShiftTypes("S" + i);
                List<JAXBElement<WeightedInteger>> minMax = instances.getMinOrMax();
                minMax.add(objectFactory.createGlobalShiftConstraintsShiftInstancesMin(Transformer.transformInteger(minInstances)));
                minMax.add(objectFactory.createGlobalShiftConstraintsShiftInstancesMax(Transformer.transformInteger(maxInstances)));
                constraints.getAverageShiftLengthOrShiftInstances().add(instances);

            }

        } else {

            GlobalShiftConstraints.ShiftInstances instances = new GlobalShiftConstraints.ShiftInstances();
            List<JAXBElement<WeightedInteger>> minMax = instances.getMinOrMax();
            minMax.add(objectFactory.createGlobalShiftConstraintsShiftInstancesMax(Transformer.transformInteger(sumInstances)));
            constraints.getAverageShiftLengthOrShiftInstances().add(instances);

        }

        shifts.setGlobalShiftConstraints(constraints);

    }

}
