package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.instances.ScheduleGenerator;
import at.ac.tuwien.dbai.ges.instances.breaks.Break;
import at.ac.tuwien.dbai.ges.instances.employee.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.instances.shift.ShiftDetails;
import at.ac.tuwien.dbai.ges.instances.tasks.Task;
import at.ac.tuwien.dbai.ges.schema.solution.Employee;
import at.ac.tuwien.dbai.ges.schema.solution.Schedule;

import java.util.TreeMap;

/**
 * Transforms the solution to the XML format.
 *
 * @author Lucas Kletzander
 */
public class SolutionTransformer extends Transformer<Schedule> {

    private final ScheduleGenerator generator;
    private final String problemFile;
    private final Schedule schedule;

    public SolutionTransformer(ScheduleGenerator generator, String problemFile) {

        this.generator = generator;
        this.problemFile = problemFile;
        schedule = new Schedule();

    }

    @Override
    public void transform() {

        schedule.setSpecification(problemFile);

        for (EmployeeSchedule row : generator.getData()) {

            Employee employee = new Employee();
            employee.setID("E" + row.getId());

            for (int i = generator.getConfig().getHistory(); i < row.getDetails().size(); i++) {

                ShiftDetails details = row.getDetails().get(i);
                if (details == null) {
                    continue;
                }

                Employee.Shift shift = new Employee.Shift();
                shift.setType("S" + generator.getShiftGenerator().getShiftType(details.getShiftInstance()));
                shift.setDay(String.valueOf(i - generator.getConfig().getHistory()));
                shift.setStartTime(Transformer.transformTimePoint(details.getStartTime()));
                shift.setEndTime(Transformer.transformTimePoint(details.getStartTime() + details.getDuration()));

                TreeMap<Integer, Object> content = new TreeMap<>();

                for (Break b : details.getBreaks()) {

                    Employee.Shift.Break br = new Employee.Shift.Break();
                    br.setType("B" + b.getType());
                    br.setStartTime(Transformer.transformTimePoint(b.getStart()));
                    br.setEndTime(Transformer.transformTimePoint(b.getEnd()));
                    content.put(b.getStart(), br);

                }

                for (Task t : details.getTasks()) {

                    int offset = (t.getDay() - i) * 24 * 60;

                    for (int j = 0; j < t.getStartList().size(); j++) {

                        Employee.Shift.Task task = new Employee.Shift.Task();
                        task.setID("TC" + t.getId());
                        task.setStartTime(Transformer.transformTimePoint(t.getStartList().get(j) + offset));
                        task.setEndTime(Transformer.transformTimePoint(t.getEndList().get(j) + offset));
                        content.put(t.getStartList().get(j) + offset, task);

                    }

                }

                content.keySet().forEach(j -> shift.getTaskOrBreak().add(content.get(j)));
                employee.getShift().add(shift);

            }

            schedule.getEmployee().add(employee);

        }

    }

    @Override
    protected Schedule getRoot() {
        return schedule;
    }

}
