package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.instances.ScheduleConfig;
import at.ac.tuwien.dbai.ges.instances.tasks.Task;
import at.ac.tuwien.dbai.ges.schema.*;

import java.util.Queue;

/**
 * Transforms the task section and the task demands of the problem to XML.
 *
 * @author Lucas Kletzander
 */
public class TaskTransformer {

    private final ScheduleConfig config;
    private final ObjectFactory objectFactory;

    public TaskTransformer(ScheduleConfig config, ObjectFactory objectFactory) {

        this.config = config;
        this.objectFactory = objectFactory;

    }

    public Tasks transformTypes(int[] reAc) {

        Tasks tasks = new Tasks();
        TaskTypes types = new TaskTypes();

        for (int i = 0; i < reAc.length; i++) {

            TaskTypes.TaskType type = new TaskTypes.TaskType();
            type.setID("T" + i);
            type.setName("T" + i);

            if (reAc[i] > 0) {
                type.setReAcquaintancePeriod(String.valueOf(reAc[i] * config.getTimeSlotLength()));
            }

            types.getTaskType().add(type);

        }

        tasks.setTaskTypes(types);
        return tasks;

    }

    public Demands transformDemands(Queue<Task> tasks) {

        Demands demands = new Demands();

        Task task;
        int id = 0;

        while ((task = tasks.poll()) != null) {

            TaskCover cover = new TaskCover();
            cover.setID("TC" + id);
            task.setId(id);
            id++;

            cover.setTask("T" + task.getType());
            cover.setLength(String.valueOf(task.getLength()));

            int day = task.getDay();
            if (task.getStart(day) - task.getWindowBefore() < 0) {
                day--;
            }

            cover.setDay(String.valueOf(day - config.getHistory()));
            cover.getStartOrEnd().add(objectFactory.createTaskCoverStart(Transformer.transformWeightedTimePoint(task.getStart(day) - task.getWindowBefore())));
            cover.getStartOrEnd().add(objectFactory.createTaskCoverEnd(Transformer.transformWeightedTimePoint(task.getEnd(day) + task.getWindowAfter())));

            if (task.getType() < config.getTaskGenerationType().ordinal()) {

                TaskCover.AllowSplit allowSplit = new TaskCover.AllowSplit();
                if (task.getType() != config.getTaskGenerationType().ordinal() - 3) {
                    allowSplit.setBreaksOnly(true);
                }
                cover.setAllowSplit(allowSplit);

            }

            CoverRequirements requirements = new CoverRequirements();
            if (task.getSkill() == -1) {
                requirements.getMinOrMax().add(objectFactory.createCoverRequirementsMin(Transformer.transformIntSkill(task.getUsage())));
            } else {
                requirements.getMinOrMax().add(objectFactory.createCoverRequirementsMin(Transformer.transformIntSkill(task.getUsage(), task.getSkill())));
            }
            requirements.getMinOrMax().add(objectFactory.createCoverRequirementsMax(Transformer.transformIntSkill(task.getUsage())));
            cover.setRequirements(requirements);

            demands.getTaskCover().add(cover);

        }

        return demands;

    }

}
