package at.ac.tuwien.dbai.ges.instances.transformer;

import at.ac.tuwien.dbai.ges.schema.*;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * Base class for transformations to the XML format.
 *
 * @author Lucas Kletzander
 */
public abstract class Transformer<T> {

    public String getDocument() {

        try {

            StringWriter writer = new StringWriter();
            JAXBContext context = JAXBContext.newInstance(getRoot().getClass());
            Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.marshal(getRoot(), writer);

            return writer.toString();

        } catch (JAXBException e) {
            e.printStackTrace();
        }

        return "Error building XML document";

    }

    public abstract void transform();

    protected abstract T getRoot();

    public void printToFile(String fileName) {

        try {

            PrintWriter writer = new PrintWriter(fileName);
            writer.println(getDocument());
            writer.close();

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public static WeightedInteger transformInteger(int integer) {

        WeightedInteger result = new WeightedInteger();
        result.setValue(BigInteger.valueOf(integer));
        return result;

    }

    public static WeightedDouble transformDouble(double d) {

        WeightedDouble result = new WeightedDouble();
        result.setValue(d);
        return result;

    }

    public static WeightedDouble transformDoubleRound(double d, int roundingMode) {

        WeightedDouble result = new WeightedDouble();
        result.setValue(new BigDecimal(d).setScale(2, roundingMode).doubleValue());
        return result;

    }

    public static WeightedTimeSpan transformTimeSpan(int timeSpan) {

        WeightedTimeSpan result = new WeightedTimeSpan();
        result.setValue(String.valueOf(timeSpan));
        return result;

    }

    public static WeightedTimeSpan transformTimeSpan(int timeSpan, double weight) {

        WeightedTimeSpan result = transformTimeSpan(timeSpan);
        result.setWeight(weight);
        return result;

    }

    public static String transformTimePoint(int timePoint) {

        String value = "";
        if (timePoint >= 24 * 60) {
            value += (timePoint / (24 * 60)) + ".";
        }
        timePoint %= (24 * 60);

        return value + String.format("%02d:%02d", timePoint / 60, timePoint % 60);

    }

    public static WeightedTimePoint transformWeightedTimePoint(int timePoint) {

        WeightedTimePoint result = new WeightedTimePoint();
        result.setValue(transformTimePoint(timePoint));
        return result;

    }

    public static WeightedIntSkill transformIntSkill(int integer) {

        WeightedIntSkill intSkill = new WeightedIntSkill();
        intSkill.setValue(BigInteger.valueOf(integer));
        return intSkill;

    }

    public static WeightedIntSkill transformIntSkill(int integer, int skill) {

        WeightedIntSkill intSkill = transformIntSkill(integer);
        intSkill.setSkill("Sk" + skill);
        return intSkill;

    }

}
